import dotenv from 'dotenv';
import { BehaviorSubject, distinctUntilChanged, filter, map, Observable, Observer, Subject, Unsubscribable } from 'rxjs';
import { WrappedMessage } from '../utils/message.ordering';
import ConsoleLogger from '../utils/log.utils';
import { AdapterBase } from '../base/adapter.base';
import { ClientObject, ConnectionState, FisMessage, TransportType, TransportMessage, TransportServiceInterface, TransmissionRole } from '../interface/interface';
import { checkRxType } from '../utils/general.utils';

dotenv.config();
export class TransmitterAdapter<T> extends AdapterBase<T> {

    constructor(clientId: string, adapterType: TransportType, transportService: TransportServiceInterface, role: TransmissionRole) {
        super(clientId, adapterType, transportService, role)
        this.console = new ConsoleLogger(`${this.adapterProfile.transportType}TransmitterAdapter`, ['adapter'])

        this.console.log({ message: `Contructing TransmitterAdapter for client: ${this.adapterProfile.clientId}` })
    }

    public subscribe(param: Observer<T> | Observable<T>): Unsubscribable | null {
        if (checkRxType(param) === `Observable`) {
            this.console.log({ message: `Is Observable` });
            // Create a new Subscription to manage unsubscription
            const subscription = (param as Observable<any>).subscribe((message: { selfId: string, payload: WrappedMessage }) => {
                this.emit(message.selfId, message.payload);
            });

            return subscription; // Return the Subscription (Unsubscribable)
        } else {
            return null;
        }
    }

    public emit(selfId: string, message: WrappedMessage): void {
        // logic here
        this.console.log({ message: `Emitting: ${message.thisMessageID} to ${this.adapterProfile.clientId}` })
        this.adapterProfile.transportService.emit({
            id: this.adapterProfile.clientId,
            dateCreated: new Date(),
            transport: this.adapterProfile.transportType,
            target: this.adapterProfile.clientId,
            source: selfId,
            payload: message
        } as TransportMessage)
    }

}


